The `heterosis_case_study` repository contains materials supporting a JASA submission by Will Landau, Jarad Niemi, and Dan Nettleton. To begin, unzip `heterosis_case_study.zip`, if applicable. Then, follow these directions to install the new methodology's software and reproduce the paper.

# System requirements

- [R](https://www.r-project.org/) version 3.2.3 or greater.
- A standard development toolkit with [GNU Make](https://www.gnu.org/software/make/), along with C and C++ compilers. For Windows platforms, a compatible version of [Rtools](https://cran.r-project.org/bin/windows/Rtools/) is enough.
- An installation of [LaTeX](https://www.latex-project.org/).
- The [`pdflatex`](https://linux.die.net/man/1/pdflatex) binary or executable.
- A [CUDA](https://developer.nvidia.com/about-cuda)-capable [NVIDIA](http://www.nvidia.com) graphics processing unit (GPU) that supports [Compute Capability](https://en.wikipedia.org/wiki/CUDA#GPUs_supported) between 3.0 and 5.0.
- The appropriate NVIDIA CUDA drivers for your system.
- A compatible version of the [CUDA Toolkit](https://developer.nvidia.com/cuda-downloads) from NVIDIA (for example, CUDA 8.0).
    
# Installing the R package dependencies

Open an R session and begin by installing the most cooperative dependency packages from [CRAN](https://cran.r-project.org/).

```r
install.packages(c(
  "actuar",
  "coda",
  "devtools",
  "doMC",
  "doParallel",
  "dplyr",
  "foreach",
  "ggplot2",
  "ggthemes",
  "gridExtra",
  "hexbin",
  "knitr",
  "plyr",
  "pracma",
  "readr",
  "reshape2",
  "snowfall",
  "testthat",
  "xtable"
))
```

The `edgeR` package from [Bioconductor](https://bioconductor.org) is also required.

```r
source("https://bioconductor.org/biocLite.R")
biocLite("edgeR")
```

Next, install the `BH` package, and then `rstan` separately afterwards.

```r
install.packages("BH")
install.packages("rstan")
```

The `rstan` package is known to have compilation issues. Some systems [require a personal `~/.R/Makevars` file](https://github.com/stan-dev/rstan/issues/447#issuecomment-325172186) containing the line,

```r
CXXFLAGS=-DBOOST_PHOENIX_NO_VARIADIC_EXPRESSION
```


# Installing the R packages for the case study

The case study paper relies on four separate R packages not on CRAN.

- [fbseq](https://github.com/wlandau/fbseq): front-end interface for the fully Bayesian method expounded in the paper.
- [fbseqOpenMP](https://github.com/wlandau/fbseqOpenMP): a backend for [fbseq](https://github.com/wlandau/fbseq) that runs the Markov chain Monte Carlo (MCMC) procedure from the paper. This package is platform agnostic and does not require a GPU, but it is slow for high-dimensional count datasets.
- [fbseqCUDA](https://github.com/wlandau/fbseqCUDA): another backend for [fbseq](https://github.com/wlandau/fbseq), but using an NVIDIA CUDA GPU to acceleraqte the MCMC. This package is platform-sensitive, but it is much faster than [fbseqOpenMP](https://github.com/wlandau/fbseqOpenMP), and it drives most of the results from the paper.
- [fbseqStudies](https://github.com/wlandau/fbseqStudies): a separate package specifically designed to reproduce the results of papers like this one.

Install [fbseq](https://github.com/wlandau/fbseq) before the other three. Local copies of these packages are already included in the `packages/` folder of the `heterosis_case_study` repository, so there is no need to download them.

```
install.packages("packages/fbseq", type = "source", repos = NULL)
install.packages("packages/fbseqOpenMP", type = "source", repos = NULL)
install.packages("packages/fbseqCUDA", type = "source", repos = NULL)
install.packages("packages/fbseqStudies", type = "source", repos = NULL)
```

If [fbseqCUDA](https://github.com/wlandau/fbseqCUDA) fails to install, or if runtime errors occur later, you may need to modify the `packages/fbseqCUDA/src/Makevars` file. For example, you may have to modify the `NVCC_FLAGS` to select a different [Compute Capability](https://en.wikipedia.org/wiki/CUDA#GPUs_supported). Or, if [CUDA](https://en.wikipedia.org/wiki/CUDA) is installed in a folder other than `/usr/local/cuda` on your system, you will need to supply the correct directory to `CUDA_HOME`.



# Learn the software

Each of the `fbseq*` packages has a `README.md` file (for example, `heterosis_case_study/packages/fbseq/README.md`), and some have vignettes (for example, `heterosis_case_study/fbseq/vignettes/model.Rmd`). These guided tutorials teach how to use the implementation of the core methodology in the manuscript. We recommend starting with `heterosis_case_study/packages/fbseq/README.md`.

# Check the installed software

Several diagnostic checks are recommended. First, run the automated unit tests for the `fbseq` package.

```r
devtools::test("packages/fbseq")
```

If no tests are skipped and all tests pass, that means that except for possibly [fbseqStudies](https://github.com/wlandau/fbseqStudies), you have all the required packages installed and they appear to be working.

[fbseqStudies](https://github.com/wlandau/fbseqStudies) has its own checks. The recommended approach is to run a scaled-down diagnostic dry run of the data analysis workflow of the case study paper.

```r
old_dir <- getwd() # Save your current directory.
dir <- tempdir() # Temporary directory.
setwd(dir) # Avoid messing up the current file system.
library(fbseqStudies)
options("fbseqStudies.scaledown" = TRUE) # Activate a scaled-down dry run.
system.time( # About 2-10 minutes, depending on the system.
  paper_case() # Reproduce the results of the case study paper.
)
list.files()
list.files("case_study_paper_figures") # Stubs of the figures in the paper.
```

The `case_study_paper_figures/` folder corresponds exactly to the `fig/` folder in this repository, the location of the figures to be loaded into the compiled manuscript.

# Reproduce the results of the paper

Finally, we are ready to reproduce the full results of the paper. The first computation below could take several hours or days, so I recommend deploying it on a cluster or server so that your job persists even after you log out.

```r
dir.create("workspace")
setwd("workspace")
options("fbseqStudies.scaledown" = FALSE)
paper_case() # Runs for hours or days.
list.files()
list.files("case_study_paper_figures")
# file.copy( # Check the figures first and then copy manually.
#   from = "case_study_paper_figures",
#   to = "../fig",
#   overwrite = TRUE,
#   recursive = TRUE
# )
```
